@@ -6,11 +6,11 @@ from .enum import enum
 from .field import field
 from .lazy_type import LazyType
 from .mutation import mutation, subscription
+from .object_type import input, interface, type
 from .permission import BasePermission
 from .private import Private
 from .scalars import ID
 from .schema import Schema
-from .type import input, interface, type
 from .union import union
 
 
