@@ -1,9 +1,11 @@
 # TODO: add links to docs
 
-from typing import List, Set
+from typing import List, Set, Union
 
 from graphql import GraphQLObjectType
 
+from strawberry.type import StrawberryType
+
 
 class NotAnEnum(Exception):
     def __init__(self):
@@ -81,10 +83,9 @@ class InvalidUnionType(Exception):
 class MissingTypesForGenericError(Exception):
     """Raised when a generic types was used without passing any type."""
 
-    def __init__(self, field_name: str, annotation):
+    def __init__(self, annotation: Union[StrawberryType, type]):
         message = (
-            f'The type "{annotation.__name__}" of the field "{field_name}" '
-            f"is generic, but no type has been passed"
+            f'The type "{repr(annotation)}" is generic, but no type has been passed'
         )
 
         super().__init__(message)
@@ -119,10 +120,10 @@ class PrivateStrawberryFieldError(Exception):
 
 
 class MultipleStrawberryArgumentsError(Exception):
-    def __init__(self, field_name: str, argument_name: str):
+    def __init__(self, argument_name: str):
         message = (
-            f"Annotation for argument `{argument_name}` on field "
-            f"`{field_name}` cannot have multiple `strawberry.argument`s"
+            f"Annotation for argument `{argument_name}` cannot have multiple "
+            f"`strawberry.argument`s"
         )
 
         super().__init__(message)
