@@ -20,9 +20,9 @@ from strawberry.union import StrawberryUnion
 from strawberry.utils.inspect import get_func_args
 
 from .field import FederationFieldParams, field as base_field
+from .object_type import FederationTypeParams, type as base_type
 from .printer import print_schema
 from .schema import Schema as BaseSchema
-from .type import FederationTypeParams, type as base_type
 
 
 def type(
