@@ -1,5 +1,6 @@
 import logging
-from typing import Any, Dict, List, Optional, Sequence, Type, Union
+import sys
+from typing import Any, Collection, Dict, List, Optional, Sequence, Type, Union
 
 from graphql import (
     ExecutionContext as GraphQLExecutionContext,
@@ -11,6 +12,7 @@ from graphql import (
 from graphql.error import GraphQLError
 from graphql.subscription import subscribe
 from graphql.type.directives import specified_directives
+from graphql.validation import ValidationRule
 
 from strawberry.custom_scalar import ScalarDefinition
 from strawberry.enum import EnumDefinition
@@ -101,9 +103,18 @@ class Schema:
     def process_errors(
         self, errors: List[GraphQLError], execution_context: ExecutionContext
     ) -> None:
+        kwargs: Dict[str, Any] = {
+            "stack_info": True,
+        }
+
+        # stacklevel was added in version 3.8
+        # https://docs.python.org/3/library/logging.html#logging.Logger.debug
+
+        if sys.version_info >= (3, 8):
+            kwargs["stacklevel"] = 3
+
         for error in errors:
-            actual_error = error.original_error or error
-            logger.error(actual_error, exc_info=actual_error)
+            logger.error(error, exc_info=error.original_error, **kwargs)
 
     async def execute(
         self,
@@ -113,6 +124,7 @@ class Schema:
         root_value: Optional[Any] = None,
         operation_name: Optional[str] = None,
         validate_queries: bool = True,
+        validation_rules: Optional[Collection[Type[ValidationRule]]] = None,
     ) -> ExecutionResult:
         # Create execution context
         execution_context = ExecutionContext(
@@ -131,16 +143,13 @@ class Schema:
             execution_context_class=self.execution_context_class,
             validate_queries=validate_queries,
             execution_context=execution_context,
+            validation_rules=validation_rules,
         )
 
         if result.errors:
             self.process_errors(result.errors, execution_context=execution_context)
 
-        return ExecutionResult(
-            data=result.data,
-            errors=result.errors,
-            extensions=result.extensions,
-        )
+        return result
 
     def execute_sync(
         self,
@@ -150,6 +159,7 @@ class Schema:
         root_value: Optional[Any] = None,
         operation_name: Optional[str] = None,
         validate_queries: bool = True,
+        validation_rules: Optional[Collection[Type[ValidationRule]]] = None,
     ) -> ExecutionResult:
         execution_context = ExecutionContext(
             query=query,
@@ -167,16 +177,13 @@ class Schema:
             execution_context_class=self.execution_context_class,
             validate_queries=validate_queries,
             execution_context=execution_context,
+            validation_rules=validation_rules,
         )
 
         if result.errors:
             self.process_errors(result.errors, execution_context=execution_context)
 
-        return ExecutionResult(
-            data=result.data,
-            errors=result.errors,
-            extensions=result.extensions,
-        )
+        return result
 
     async def subscribe(
         self,
